#!/usr/bin/ruby

require 'kconv'
require 'standard/swrba_command'          # Command Class
require 'standard/swrba_logger'           # Logger Class
require 'standard/swrba_def'              # Common Definition Class
require 'standard/swrba_input'            # Input Information File Analysis Class

#=ESX Server Operation Parts
#Version::   1.0
#Copyright:: Copyright (C) FUJITSU LIMITED 2010.
#
#=== Function Contents
#* Operate ESX Server
#
#=== Development history
#* 2010-07-20
#  * Create.
#

# Const value

Snap_in = "Add-PSSnapin VMware.VimAutomation.Core"
Initial = "Initialize-VIToolkitEnvironment.ps1"

Connect_string = "$user=\"%s\";$password=\"%s\";\n" +
                  "$vmserver=Connect-VIServer %s -User $user -Password $password"
Disconnect_string = "Disconnect-VIServer -Server $vmserver -Confirm:$False"

VMShutdown = ";$vmhost=\"%s\";$vm=Get-VM -Location"+
              " (Get-VMHost $vmhost);$vm|Where-Object {$_.PowerState -eq 'PoweredOn'}"+
              "|foreach { Shutdown-VMGuest -VM $_ -Confirm:$False; if ($? -eq $False)"+ 
              " { Stop-VM -VM $_ -Confirm:$False}};$x = 300;while ($x -gt 0)"+ 
              " {$oncount=0;$vm=Get-VM -Location (Get-VMHost $vmhost);$vm |Where-Object"+ 
              " {$_.PowerState -eq 'PoweredOn'}|foreach {$oncount++};"+
              " if ($oncount -eq 0){break;}Start-Sleep -Seconds 10;$x=$x-10;};"+
              " Get-VM -Location (Get-VMHost $vmhost)|Where-Object"+ 
              " {$_.PowerState -eq 'PoweredOn'}|foreach {Stop-VM -VM $_ -RunAsync:$False -Confirm:$False};"
              
Trap_string = "trap{'';exit 161}"
Operation_Kind="VMshutdown"

# define hash
Operation_esx = {
'VMshutdown' => VMShutdown
}

# Logger Class Instance
logger = Swrba_Logger.new(Operation_Kind, Swrba_Def::SWRBA_FILE_LOG)

logger.warn(":[begin]")

# Input Information File Analysis Class
input = Swrba_Input.new

# Initialize
option = []

# Get Option Information
while data = ARGV.shift
  option.push(data)
end

# Check Parameter Number
if option.size != 2
  logger.warn("parameter check error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Check Option Form
if option[0] != "-input"
  logger.warn("option check error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Check File Exist
if File.exist?(option[1]) == false
  logger.warn("file exist error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_ARGUMENT
end

# Input File Name
logger.warn("Input File Name = " + option[1])

# Get Input Information
begin
  hostname     = input.input_analysis(option[1], "hostname")
  ipaddress    = input.input_analysis(option[1], "ipaddress")
  esxname     = input.input_analysis(option[1], "esxname")
  execusername = input.input_analysis(option[1], "execusername")
  execpassword = input.input_analysis(option[1], "execpassword")
rescue
  logger.warn("input analysis error")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_RETRY
end

# Priority to IP Address
if ipaddress != nil && ipaddress != ""
  hostname = ipaddress
end

# Check Required
if hostname == nil
  logger.warn("hostname None")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_INPUT_DATA
end

if esxname == nil
  logger.warn("esxname None")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_INPUT_DATA
end

if execusername == nil || execusername == ""
  logger.warn("execusername None")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_INPUT_DATA
end

if execpassword == nil || execpassword == ""
  logger.warn("execpassword None")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_INPUT_DATA
end

# make PowerShell Script 

# get an environment variable to name a PowerShell Script file
begin
  tmp_path = ENV["ITPM_HOME"] + "/RBAOPE/work/temp/" + ENV["SWRBA_PROCESS_ID"] + "_" + ENV["SWRBA_ACTIVITY_ID"] + ".ps1"
  logger.warn(tmp_path)
rescue => exc
  logger.warn("Acquisition error of environment variable")
  logger.warn(exc)
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_OTHER
end

# replace '\\' with '/'
tmp_file = tmp_path.gsub('\\','/')

esx_ope = Operation_esx[Operation_Kind]
if esx_ope == nil
  logger.warn("Operation invalid")
  logger.warn(":[error]")
  exit Swrba_Def::ERROR_INPUT_DATA
end

begin
  File.open(tmp_file,"w" ) {|file|
  file.puts(Snap_in)
  file.puts(Initial)
  file.puts(sprintf(Connect_string,execusername,execpassword,hostname))
  file.puts(sprintf(esx_ope,esxname))
  file.puts(Disconnect_string)
  file.puts(Trap_string)
}
rescue => exc
  logger.warn("Generation error of PowerShell Script")
  logger.warn(exc)
  logger.warn(":[error]")
  
  begin
    if File.exist?(tmp_file) == true
     File.delete(tmp_file)
    end
  rescue => exc
    logger.warn("Delete file error of PowerShell Script")
    logger.warn(exc)
    logger.warn(":[error]")
  end
  
  exit Swrba_Def::ERROR_OTHER
end

# start PowerShell

# Command Line Instance
command = Swrba_Command.new

commandline = "powershell -command \"" + tmp_file+";exit $LASTEXITCODE\""


begin
  command.run("localhost", execusername, execpassword, execusername, execpassword, commandline)
rescue
end
if command.return_code != 0
  # Standard Error Output
  STDERR.puts command.stderr.toutf8
  logger.warn("esx operation error")
  logger.warn(":[error]")
  
  begin
    if File.exist?(tmp_file) == true
      File.delete(tmp_file)
    end
  rescue => exc
    logger.warn("Delete file error of PowerShell Script")
    logger.warn(exc)
    logger.warn(":[error]")
  end
  
  exit Swrba_Def::ERROR_RETRY
end

# Standard Output
STDERR.puts "Success"

# Delete PowerShell Script
begin
  if File.exist?(tmp_file) == true
    File.delete(tmp_file)
  end
rescue => exc
  logger.warn("Delete file error of PowerShell Script")
  logger.warn(exc)
  logger.warn(":[error]")
end

logger.warn(":[end]")

exit Swrba_Def::NORMAL_CODE
